function LWDF_q = quantLWDF(LWDF, bw, mode)
%quantLWDF	Quantize filter coefficients of given LWDF filter structure
%           assuming a fractional binary format with a decimal range
%           of [-1,1[ .
%	LWDF_q = quantLWDF(LWDF, bw, mode) returns the LWDF filter coefficients
%            rounded with mode to a fractional format of bitwidth bw.
%            Parameter mode must be one of the following ('trunc'|'round')
%	Here LWDF is a structure that should contain the fields
%		LWDF.wdaCodes
%		LWDF.gamma
%	LWDF.wdaCodes is an [2x?] character array, which is
%	described in 'Hs2LWDF.m', as well as in 'showLWDF.m'. 
%	LWDF.gamma contains the coefficients belonging to the wdaCodes.
%	The returned structure LWDF_q contains the fields of the original
%	structure where wdaCodes is left unchanged, while gamma contains the
%	quantised coefficient values.

% (c) Christian Zeintl, 
%     University of Applied Sciences Upper Austria, May 2017

wdaCodes = LWDF.wdaCodes;
gamma	 = LWDF.gamma;
bw = floor(bw);
mode = lower(mode);

% Assertions and input range checks
nonFrac = (gamma >= 1) + (gamma < -1);
nonFrac = sum(nonFrac(:));
assert(nonFrac == 0, 'Input filter coefficients exceeding fractional range [-1,1[ !');
assert(bw > 1, 'Bitwidth must be greater than 1!');
if (~strcmp(mode,'round') && ~strcmp(mode, 'trunc') && ~strcmp(mode, 'floor') && ~strcmp(mode, 'ceil'))
    error('Mode must be one of the following (''trunc'' | ''round'' | ''floor'' | ''ceil'')')
end

% actual quantisation
lsb = 2^-(bw-1);

if strcmp(mode, 'round')
    gamma_q = round(gamma / lsb) * lsb;
elseif strcmp(mode, 'floor')
    gamma_q = floor(gamma / lsb) * lsb;
elseif strcmp(mode, 'ceil')
    gamma_q = ceil(gamma / lsb) * lsb;
else
    if gamma >= 0
        gamma_q = floor(gamma / lsb) * lsb;
    else
        gamma_q = ceil(gamma / lsb) * lsb;
    end
end

% write resulting structure
LWDF_q.wdaCodes  = wdaCodes;
LWDF_q.gamma     = gamma_q;
